///////////////////////////////////////////////////////////////////////////////
//
// ScreenColor.h
//
//
// 
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SCREENCOLOR_H
#define SCREENCOLOR_H

#include <gdi.h>
class TResourceReader;


///////////////////////////////////////////////////////////////////////////////
// MDynamicColor
///////////////////////////////////////////////////////////////////////////////
class MDynamicColor
	{
public:
	virtual TRgb Color(TInt aStep) const = 0;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// TFade
///////////////////////////////////////////////////////////////////////////////
// Base class which provides a basic define of a fade in terms of a linear
// color change over a series of steps.
///////////////////////////////////////////////////////////////////////////////
class TFade
	{
public:
	enum TType
		{
		ENoFade,
		EDarker,
		EBrighter
		};
public:
	static TRgb FadeColor(TType aType, TRgb aFadeFrom, TRgb aFadeBy, TInt aSteps=1);
public:
	TFade();
	TFade(TInt aSteps, TRgb aFadeFrom, TRgb aFadeTo);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
public:
	virtual void Set(TInt aSteps, TRgb aFadeFrom, TRgb aFadeTo);
	inline TType FadeType() const { return TType(iFadeType); }
	inline TInt Steps() const { return TInt(iSteps); }
	inline TRgb ColorStep() const { return iColStep; }
protected:
	TUint8 iSteps;
	TRgb iColStep;
	TUint8 iFadeType;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// TFadedColor
///////////////////////////////////////////////////////////////////////////////
// Fade from a starting color to an end color.
// Also supports using a single fixed color instead of a fade.
///////////////////////////////////////////////////////////////////////////////
class TFadedColor : public TFade, public MDynamicColor
	{
public:
	TFadedColor();
	TFadedColor(TRgb aFixedColor);
	TFadedColor(TInt aSteps, TRgb aFadeFrom, TRgb aFadeTo);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
public:
	void Set(TRgb aFixedColor);
	virtual void Set(TInt aSteps, TRgb aFadeFrom, TRgb aFadeTo);
	virtual TRgb Color(TInt aStep) const;
protected:
	TRgb iFixedColor;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// TMultiFadedColor
///////////////////////////////////////////////////////////////////////////////
// Allows several fades to be combined in one sequence.
// e.g. Fade from dark to light back to dark.
///////////////////////////////////////////////////////////////////////////////
class TMultiFadedColor : public MDynamicColor
	{
public:
	enum { KMaxFades = 3 };
public:
	inline TMultiFadedColor() {}
	void InternalizeL(TResourceReader&);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;
public:
	virtual TRgb Color(TInt aStep) const;
	TInt Steps() const;
	void Set(TInt aStepsForEachFade, const TRgb& aStartCol, const TRgb& aCol2, const TRgb& aCol3, const TRgb& aEndCol);
	inline const TFadedColor& operator[](TInt aIndex) const { return iFades[aIndex]; }
	inline TFadedColor& operator[](TInt aIndex) { return iFades[aIndex]; }
private:
	TFixedArray<TFadedColor, KMaxFades> iFades;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
