///////////////////////////////////////////////////////////////////////////////
//
// ScreenFurniture.h
//
//
// 
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SCREENFURNITURE_H
#define SCREENFURNITURE_H

#include <fbs.h>
#include <gulalign.h>
#include "Screen.h"
#include "ScreenUtility.h"

#include "Sprites.h"

class TTrigUnit;
class TResourceReader;
class RReadStream;
class RWriteStream;

///////////////////////////////////////////////////////////////////////////////
// CScreenBackground
///////////////////////////////////////////////////////////////////////////////
class CScreenBackground : public CBase
	{
public:
	CScreenBackground();
	virtual ~CScreenBackground();
	void ConstructL();
	void InternalizeL(TResourceReader&);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;
	void Draw(CBitmapContext&);
	void Reset();
	void SetSize(const TSize&);
	void SetBaseColor(const TRgb&);
	TInt AddBitmapL(CFbsBitmap* aBitmap, TInt aMaskIndex=-1);
	TInt AddBitmapL(const TPoint& aPos, CFbsBitmap* aBitmap, TInt aMaskIndex=-1);
	TInt AddMaskL(CFbsBitmap* aBitmap);
private:
	TSize iSize;
	TRgb iBaseColor;
	struct TBitmap
		{
		TPoint iPos;
		CFbsBitmap* iBitmap;
		TInt iMaskIndex;
		};
	CArrayFix<TBitmap>* iBitmaps;
	CArrayPtr<CFbsBitmap>* iMasks;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenBox
///////////////////////////////////////////////////////////////////////////////
class CScreenBox : public CBase
	{
public:
	CScreenBox(CScreenBitmap& aScreen);
	void ConstructL(const TRect&);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void SetRectL(const TRect&);
	virtual void Draw(CGraphicsContext&);
	void SetAlignment(CGraphicsContext::TTextAlign);
	void SetFillStyle(const TScreenFillStyle&);
public:
	inline const TRect& Rect() const { return iRect; }
	inline const TScreenFillStyle& FillStyle() const { return iFillStyle; }
protected:
	TRect iRect;
	CGraphicsContext::TTextAlign iAlignment;
	TScreenFillStyle iFillStyle;
	CScreenBitmap& iScreen;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenBoxList
///////////////////////////////////////////////////////////////////////////////
// Alignment specifies the alignment of lines within the list rect.
// Height of each line is taken from the list element.
// Total height will grow as items are added.
// Width of each line is taken from the widest list element.
// Width will grow as wider items are added.
// SetRectL() will override these defaults, and elements will be resized to fit
//  within the rect dimensions, relative heights will be maintained.
// SetPositionL() may be used to position the box list on screen without
//  altering the box's size.
// ResizeL() may be used to set the box's size without altering it's position.
///////////////////////////////////////////////////////////////////////////////
class CScreenBoxList : public CScreenBox
	{
public:
	CScreenBoxList(CScreenBitmap& aScreen);
	virtual ~CScreenBoxList();
	void ConstructL(const TPoint&);
	virtual void SetRectL(const TRect&);
	virtual void Draw(CGraphicsContext&);
	virtual void SetPositionL(const TPoint&);
	virtual void ResizeL(const TSize&);
public:
	inline TInt Count() const { return iList.Count(); }
	inline CScreenBox& operator[](TInt aIndex) { return *iList[aIndex]; }
	inline const CScreenBox& operator[](TInt aIndex) const { return *iList[aIndex]; }
	virtual void AddBoxL(CScreenBox* aBox, const TSize& aBoxSize, TInt aIndex=-1);
	virtual CScreenBox* RemoveBox(TInt aIndex);
	virtual void Reset();
protected:
	void ShiftLines(TInt aIndex, TInt aShiftBy, TInt aBoxWidth);
protected:
	RPointerArray<CScreenBox> iList;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenTextBox
///////////////////////////////////////////////////////////////////////////////
class CScreenTextBox : public CScreenBox
	{
public:
	CScreenTextBox(CScreenBitmap& aScreen);
	virtual ~CScreenTextBox();
	void ConstructL(const TRect&, const TFontSpec&);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void SetRectL(const TRect&);
	virtual void SetFontL(const TFontSpec&);
	virtual void SetTextL(const TDesC&);
	virtual void Draw(CGraphicsContext&);
	void SetPenStyle(const TScreenPenStyle&);
public:
	inline const CFont* Font() const { return iFont; }
	inline const TFontSpec& FontSpec() const { return iFontSpec; }
protected:
	MGraphicsDeviceMap& FontDevice();
	void LoadFont();
	void ReleaseFont();
	void DrawText(CGraphicsContext&, const TDesC&, const TRect&);
protected:
	HBufC* iText;
	TFontSpec iFontSpec;
	TScreenPenStyle iPenStyle;
private:
	CFont* iFont;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenNumberBox
///////////////////////////////////////////////////////////////////////////////
class CScreenNumberBox : public CScreenTextBox
	{
public:
	CScreenNumberBox(CScreenBitmap& aScreen);
	void SetNumberL(TInt);
	void SetNumberL(TUint);
	void SetNumberL(const TTrigUnit&);
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenMultilineTextBox
///////////////////////////////////////////////////////////////////////////////
class CScreenMultilineTextBox : public CScreenTextBox
	{
public:
	CScreenMultilineTextBox(CScreenBitmap& aScreen);
	void ConstructL(const TRect&, const TFontSpec&, TInt aNumLines);
	void SetNumLinesL(TInt aLines, TBool aResize);
	void SetLineHeightL(TInt aHeight);
	TInt LineHeight() const;
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void SetFontL(const TFontSpec&);
	virtual void Draw(CGraphicsContext&);
private:
	TInt iNumLines;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenScrollingTextBox
///////////////////////////////////////////////////////////////////////////////
class CScreenScrollingTextBox : public CScreenTextBox
	{
public:
	CScreenScrollingTextBox(CScreenBitmap& aScreen);
	void ConstructL(const TRect&, const TFontSpec&, TInt aScrollRate=1);
	void SetScrollRate(TInt);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void Draw(CGraphicsContext&);
private:
	TInt iScrollRate;
	TInt iScroll;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CScreenSpriteBox
///////////////////////////////////////////////////////////////////////////////
class CScreenSpriteBox : public CScreenBox
	{
public:
	CScreenSpriteBox(CScreenBitmap&, CGameSpriteTable&);
	virtual ~CScreenSpriteBox();
	void ConstructL(const TPoint&, TSpriteId, CScreenTextBox*);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void SetRectL(const TRect&);
	virtual void Draw(CGraphicsContext&);
	inline RGameSprite& Sprite() { return iSprite; }
	inline CScreenTextBox& TextBox() { return *iTextBox; }
	void SetTextBox(CScreenTextBox*);
protected:
	RGameSprite iSprite;
	CScreenTextBox* iTextBox;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
