///////////////////////////////////////////////////////////////////////////////
//
// ScreenUtility.h
//
//
// 
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SCREENUTILITY_H
#define SCREENUTILITY_H

#include <fbs.h>
#include <gulalign.h>
class TResourceReader;
class RReadStream;
class RWriteStream;

///////////////////////////////////////////////////////////////////////////////
// TBitmapDrawer
///////////////////////////////////////////////////////////////////////////////
class TBitmapDrawer
	{
public:
	static void DrawBitmap(CBitmapContext& aGc, const CFbsBitmap& aBitmap, const TRect& aRect, const TRect* aSrcRect=NULL);
	static void BlitBitmap(CBitmapContext& aGc, const CFbsBitmap& aBitmap, const TPoint& aPos, const TRect* aSrcRect=NULL);
public:
	TBitmapDrawer();
	TBitmapDrawer(const CFbsBitmap& aBitmap, const CFbsBitmap* aMask=NULL);
	void Set(const CFbsBitmap& aBitmap, const CFbsBitmap* aMask=NULL);
	void Reset();
	void SetAlignment(TGulAlignmentValue aAlign);
	void Draw(CBitmapContext& aGc, const TRect& aRect, const TRect* aSrcRect=NULL) const;
	void Blit(CBitmapContext& aGc, const TPoint& aPos, const TRect& aSrcRect) const;
private:
	TGulAlignment iAlignment;
	const CFbsBitmap* iBitmap;
	const CFbsBitmap* iMaskBitmap;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TScreenObjectStyle
///////////////////////////////////////////////////////////////////////////////
class TScreenObjectStyle
	{
public:
	virtual void Set(CGraphicsContext&) = 0;
	virtual void UnSet(CGraphicsContext&);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
protected:
	TScreenObjectStyle();
	TScreenObjectStyle(TInt, TRgb);
public:
	TInt iStyle;
	TRgb iColor;
	TInt8 iDrawMode;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TScreenPenStyle
///////////////////////////////////////////////////////////////////////////////
class TScreenPenStyle : public TScreenObjectStyle
	{
public:
	TScreenPenStyle();
	TScreenPenStyle(CGraphicsContext::TPenStyle, TRgb);
	virtual void Set(CGraphicsContext&);
	virtual void UnSet(CGraphicsContext&);
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
public:
	TSize iSize;
	const CFont* iFont;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TScreenFillStyle
///////////////////////////////////////////////////////////////////////////////
class TScreenFillStyle : public TScreenObjectStyle
	{
public:
	TScreenFillStyle();
	TScreenFillStyle(CGraphicsContext::TBrushStyle, TRgb);
	virtual void Set(CGraphicsContext&);
	virtual void UnSet(CGraphicsContext&);
public:
	const CFbsBitmap* iBitmap;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
