///////////////////////////////////////////////////////////////////////////////
//
// AsteroidState.h
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef ASTEROIDSTATE_H
#define ASTEROIDSTATE_H

#include "ObjectStates.h"

///////////////////////////////////////////////////////////////////////////////
// CGameObjectAsteroidState
///////////////////////////////////////////////////////////////////////////////
// Messages Handled:
//  KMsgId_ObjectUpdate
//  KMsgId_ObjectCollision
//
// State for handling asteroid objects.
// Asteroids are randomly generated based upon parameters specifed in the
// resource file.
//
///////////////////////////////////////////////////////////////////////////////
class CGameObjectAsteroidState : public CGameObjectMotiveSolidState
	{
public:
	CGameObjectAsteroidState(CGameObjectController& aController);
	virtual ~CGameObjectAsteroidState();
	virtual TUidType Type() const;
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	void NewAsteroidsL(TInt aNumAsteroids, const TCoOrds& aPos, const TCoOrds& aPosDelta,
		const TTrigUnit& aAveMass, TDirection aFacing, const TTrigUnit& aTotalMomentum);
	inline TTrigUnit MaxMass() const { return iMaxMass; }
protected:
	virtual TGameMessage::TResponse OnEnterL(const TGameMessage& aMessage);
	virtual TGameMessage::TResponse OnCollisionL(const TGameMessage& aMessage);
	virtual TGameMessage::TResponse ChangeObjectSpriteL(const TGameMessage& aMessage);
private:
	void SplinterL(const TGameMessage&);
	CGameSpriteData* ObjectSpritePointsL(const CGameObject& aObject);
private:
	TGameObjectEmitter iChildren;
	TFadedColor iAsteroidColor;
	TTrigUnit iCollisionMassFactor;  // Proportion of mass retained by splinters after collision.
	TTrigUnit iMinMass;              // The minimum mass an asteroid may have.
	TTrigUnit iMaxMass;              // The maxmum mass an asteroid may have.
	TInt iMinRadius;
	TInt iMaxRadius;
	TInt iSplinterMassDeviation;
	TInt iSplinterMassDeviationRange;
	TInt iPointValBase;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

TPoint operator*(const TPoint& aPoint, TInt aInt);
TPoint operator/(const TPoint& aPoint, TInt aInt);
TPoint operator+=(TPoint& aPoint, TInt aInt);
TPoint operator-=(TPoint& aPoint, TInt aInt);
TPoint operator*=(TPoint& aPoint, TInt aInt);
TPoint operator/=(TPoint& aPoint, TInt aInt);


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
