///////////////////////////////////////////////////////////////////////////////
//
// GameHighScoreTable.h
//
//
// High score table as a simple sorted array supporting persistance.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef GAMEHIGHSCORETABLE_H
#define GAMEHIGHSCORETABLE_H

#include <e32base.h>
#include <s32strm.h>

///////////////////////////////////////////////////////////////////////////////
// CGameHighScoreTable
///////////////////////////////////////////////////////////////////////////////
class CGameHighScoreTable : public CBase
	{
public:
	enum { KMaxNameLen = 10, KCharUnderscore = '_' };
	typedef TBuf<KMaxNameLen> TName;
public:
	static CGameHighScoreTable* NewL(TInt aMaxScores);
	static CGameHighScoreTable* NewL(RReadStream&);
	virtual ~CGameHighScoreTable();
	void ExternalizeL(RWriteStream&) const;
	void Clear();
	void SetMaxCountL(TInt);
	void Add(const TDesC& aName, TInt32 aPoints, TUint8 aWave);  // May reorder list !!
	void Set(TInt aIndex, const TDesC& aName);
	TInt FindFirstLower(TInt32 aPoints) const;
	TInt MaxCount() const;
	const TDesC& Name(TInt aIndex) const;
	TInt32 Points(TInt aIndex) const;
	TUint32 Wave(TInt aIndex) const;
private:
	void ConstructL(RReadStream&);
	void FormatName(TName&);
	void ActualIndex(TInt& aIndex) const;
private:
	class TScore
		{
		public:
			TName iName;
			TInt32 iPoints;
			TUint8 iWave;
		};
	static TInt PointsOrder(const TScore&,const TScore&);
	RArray<TScore> iScores;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
