///////////////////////////////////////////////////////////////////////////////
//
// GameModelObserver.h
//
//
// 
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef GAMEMODELOBSERVER_H
#define GAMEMODELOBSERVER_H


#include "Messages.h"

class CGameModel;
///////////////////////////////////////////////////////////////////////////////
// MGameModelObserver
///////////////////////////////////////////////////////////////////////////////
// Permits observing of the game.
// This mixin is used by the app UI to drive the model through a series of
// 'waves' of asteroids.
///////////////////////////////////////////////////////////////////////////////
class MGameModelObserver
	{
public:
	enum TNotification
		{
		EStartGame,
		EGameTick,
		EStopGame,
		EResumeGame
		};
	virtual void OnGameEventL(CGameModel&, TNotification) = 0;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// MGameModelPlayerInput
///////////////////////////////////////////////////////////////////////////////
// Known player input codes.
///////////////////////////////////////////////////////////////////////////////
enum TPlayerKey
	{
	EPlayerKeyThrust    = 0x01,
	EPlayerKeyStop      = 0x02,
	EPlayerKeyTurnLeft  = 0x04,
	EPlayerKeyTurnRight = 0x08,
	EPlayerKeyShoot     = 0x10,
	EPlayerKeyBegin     = 0x20,
	EPlayerKeyEsc       = 0x40,
	EPlayerKeyActivate  = 0x80
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// MGameModelPlayerInput
///////////////////////////////////////////////////////////////////////////////
// The mechanism for providing player input to the game model.
// The game model will call GetPlayerInput() at the start of each game tick.
// ResetPlayerInput() will be called when the model has a need to clear any
// outstanding input.
// PlayerAddressL() allows the model to identify the object that messages for
// the player object should be passed to.
///////////////////////////////////////////////////////////////////////////////
class MGameModelPlayerInput
	{
public:
	class TPlayerKeyState
		{
	public:
		inline TPlayerKeyState() : iKeyState(0U) {}
		inline TBool IsPressed(TPlayerKey aKey) const       { return iKeyState & TUint16(aKey); }
		inline void SetPressed(TPlayerKey aKey, TBool aSet) { if(aSet) iKeyState |= TUint16(aKey); else iKeyState &= TUint16(~aKey); }
	public:
		TUint16 iKeyState;
		};
public:
	virtual TGameMessage::TAddress PlayerAddressL(CGameModel&) = 0;
	virtual void GetPlayerInput(CGameModel&, TPlayerKeyState&) = 0;
	virtual void ResetPlayerInput(CGameModel&) = 0;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif


