///////////////////////////////////////////////////////////////////////////////
//
// GameResourceUtils.cpp
//
///////////////////////////////////////////////////////////////////////////////
#include <barsc.h>
#include <barsread.h>

#include "GameResourceUtils.h"

#include "SpriteBitmap.h"
#include "Objects.h"
#include "GameModel.h"

///////////////////////////////////////////////////////////////////////////////
// GameResourceUtils
///////////////////////////////////////////////////////////////////////////////
void GameResourceUtils::OpenResourceLC(RResourceFile& aFile, RFs &aFs, const TDesC& aName, TInt aSignature)
	{ // static
	aFile.OpenL(aFs, aName);
	CleanupClosePushL(aFile);
	aFile.ConfirmSignatureL(aSignature);
	aSignature = aFile.Offset();
	if(!aSignature)
		{
		// There is no NAME statement at the start of the resource file.
		User::Leave(KErrBadName);
		}
	}

///////////////////////////////////////////////////////////////////////////////
void GameResourceUtils::ReadResourceLC(const RResourceFile& aFile, TInt aResourceId, TResourceReader& aReader)
	{ // static
	aReader.SetBuffer( aFile.AllocReadLC(aResourceId) );
	}

///////////////////////////////////////////////////////////////////////////////
TPoint GameResourceUtils::ReadTPoint(TResourceReader& aReader)
	{
	TPoint point;
	point.iX = aReader.ReadInt32();
	point.iY = aReader.ReadInt32();
	return point;
	}

///////////////////////////////////////////////////////////////////////////////
TSize GameResourceUtils::ReadTSize(TResourceReader& aReader)
	{
	TSize size;
	size.iWidth = aReader.ReadInt32();
	size.iHeight = aReader.ReadInt32();
	return size;
	}

///////////////////////////////////////////////////////////////////////////////
TTrigUnit GameResourceUtils::ReadTTrigUnit(TResourceReader& aReader)
	{ // static
	return TTrigUnit(aReader.ReadReal64());
	}

///////////////////////////////////////////////////////////////////////////////
TTrigXY GameResourceUtils::ReadTTrigXY(TResourceReader& aReader)
	{ // static
	TTrigXY xy;
	xy.iX = GameResourceUtils::ReadTTrigUnit(aReader);
	xy.iY = GameResourceUtils::ReadTTrigUnit(aReader);
	return xy;
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////