///////////////////////////////////////////////////////////////////////////////
//
// SpriteTable.h
//
//
// Sprite data classes
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SPRITETABLE_H
#define SPRITETABLE_H

#include "GameContainer.h"
#include "Sprites.h"


///////////////////////////////////////////////////////////////////////////////
// MGameSpriteObject
///////////////////////////////////////////////////////////////////////////////
// Sprite base class.
///////////////////////////////////////////////////////////////////////////////
class MGameSpriteObject : public MGameStreamable
	{
public:
	virtual ~MGameSpriteObject() {}
	virtual TInt FrameCount(CGameSpriteData* aSpriteData) const = 0;
	virtual TSize FrameSize(TInt aFrame, CGameSpriteData* aSpriteData) const = 0;
	virtual void DrawFrame(TInt aFrame, CBitmapContext& aGc, const TPoint& aPos, CGameSpriteData* aSpriteData, MMotionBounds* aBounds) const = 0;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// CGameSpriteTable
///////////////////////////////////////////////////////////////////////////////
// Sprite table. Container for all sprite objects.
///////////////////////////////////////////////////////////////////////////////
class CGameSpriteTable : public CGameContainer<MGameSpriteObject>
	{
public:
	friend class RGameSprite;
public:
	// See CGameContainer for element accessors.
	inline CGameSpriteTable();
	inline TBool Exists(TSpriteId aId) const;
	inline TSpriteId AddSpriteL(MGameSpriteObject* aSprite);
	inline void InsertSpriteL(TSpriteId aId, MGameSpriteObject* aNewSprite);
	inline MGameSpriteObject* RemoveSprite(TSpriteId aId);
	inline void ReplaceSprite(TSpriteId aId, MGameSpriteObject* aNewSprite);
protected:
	virtual MGameStreamable* NewByTypeLC(const TUidType& aType);
	};

///////////////////////////////////////////////////////////////////////////////
inline CGameSpriteTable::CGameSpriteTable()
	{}
inline TBool CGameSpriteTable::Exists(TSpriteId aId) const
	{ return CGameContainer<MGameSpriteObject>::Exists(aId); }
inline TSpriteId CGameSpriteTable::AddSpriteL(MGameSpriteObject* aSprite)
	{ return CGameContainer<MGameSpriteObject>::AddL(aSprite); }
inline void CGameSpriteTable::InsertSpriteL(TSpriteId aId, MGameSpriteObject* aNewSprite)
	{ CGameContainer<MGameSpriteObject>::InsertL(aId, aNewSprite); }
inline MGameSpriteObject* CGameSpriteTable::RemoveSprite(TSpriteId aId)
	{ return CGameContainer<MGameSpriteObject>::Remove(aId); }
inline void CGameSpriteTable::ReplaceSprite(TSpriteId aId, MGameSpriteObject* aNewSprite)
	{ CGameContainer<MGameSpriteObject>::ReplaceL(aId, aNewSprite); }

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif



