///////////////////////////////////////////////////////////////////////////////
//
// Sprites.h
//
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SPRITES_H
#define SPRITES_H

#include <e32base.h>
#include "Trigonometry.h"
class MDynamicColor;
class MMotionBounds;
class CGameSpriteTable;
class CGraphicsContext;
class CBitmapContext;
class TResourceReader;
class RReadStream;
class RWriteStream;

typedef TInt TSpriteId; // ID of sprite in sprite table.


///////////////////////////////////////////////////////////////////////////////
// CGameSpriteData
///////////////////////////////////////////////////////////////////////////////
// Contains object specific sprite data, (e.g. list of asteroid points).
///////////////////////////////////////////////////////////////////////////////
class CGameSpriteData : public CBase
	{
public:
	CGameSpriteData();
	CGameSpriteData(TInt aVertexArrayGranularity);
	virtual ~CGameSpriteData();
	void InternalizeL(TResourceReader&);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;
public:
	// Plotting functions.
	void PlotPoints(CGraphicsContext& aGc, const TPoint& aObjectPosition, MMotionBounds* aBounds);
	void PlotPolygon(CGraphicsContext& aGc, const TPoint& aObjectPosition, MMotionBounds* aBounds);
	TPoint ProcessVertex(const TPoint& aStart, const TPoint& aVertex, MMotionBounds* aBounds);
public:
	// Vertices.
	TRect Bounds() const;
	TInt Count() const;
	const TPoint& Point(TInt) const;
	void SetPointL(TInt, const TPoint&);
	void InsertPointL(TInt, const TPoint&);
	void DeletePoint(TInt);
	TInt AppendPointL(const TPoint&);
	void ResetPoints();
public:
	// Color.
	MDynamicColor* Color();
	void SetColor(MDynamicColor* aColor, TBool aOwned, TInt aGradient=1);
public:
	// Point calculation functions.
	void AddRadialPointsL(TTrigUnit aRadius, const TPoint& aCentreOffsetFromTl); // Plotted relative to TL.
	void AddTrianglePointsL(TTrigUnit aRadius); // Plotted relative to TL.
private:
	RArray<TPoint> iPoints;
	MDynamicColor* iColor;
	TInt iColorGradient;
	TUint iFlags;
	enum TFlags { EFlagOwnsColor=0x01 };
public:
	TTrigUnit iTrigUnit;   // For sprite<-->state interaction.
	TTrigXY iTrigXY;       // For sprite<-->state interaction.
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// RGameSprite
///////////////////////////////////////////////////////////////////////////////
// Main object sprite API.
///////////////////////////////////////////////////////////////////////////////
class RGameSprite
	{
public:
	// Construction and assignment.
	RGameSprite();
	RGameSprite(CGameSpriteTable& aSpriteTable);

	// Persistance.
	void InternalizeL(TResourceReader&);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;

	// Sprite context.
	void SetSpriteTable(CGameSpriteTable& aSpriteTable);
	void OpenL(TSpriteId aId);
	void Close();

	// Instance Data.
	TInt Frame() const;
	void SetSpriteFrame(TInt);
	void SetSpriteData(CGameSpriteData* aSpriteData, TBool aOwned);

	// Attribute acessors.
	TSpriteId Id() const;
	TUidType Type() const;
	CGameSpriteTable* SpriteTable() const;
	CGameSpriteData* Data() const;
	TInt FrameCount() const;
	TSize FrameSize() const;

	// Operations.
	void DrawFrame(CBitmapContext& aGc, const TPoint& aPos, MMotionBounds* aBounds=NULL) const;

private:
	void AssertOpen() const;
private:
	TSpriteId iId;
	CGameSpriteTable* iSpriteTable;
	CGameSpriteData* iData;
	TBool iOwnsData;
	TInt iSpriteFrame;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
