///////////////////////////////////////////////////////////////////////////////
//
// Physics.cpp
//
//
//
//
//
///////////////////////////////////////////////////////////////////////////////
#include "Physics.h"


///////////////////////////////////////////////////////////////////////////////
// TVector
///////////////////////////////////////////////////////////////////////////////
void TVector::SetUnit()
	{
	const TTrigUnit u(1);
	TTrigXY::Set(u, u);
	}

///////////////////////////////////////////////////////////////////////////////
void TVector::Normalize()
	{
	if((iX != 0) || (iY != 0) )
		{
		TTrigUnit n(iX.Pow2());
		n += iY.Pow2();
		TTrigXY::operator/=(n);
		}
	}

///////////////////////////////////////////////////////////////////////////////
TDirection TVector::Direction() const
	{
	// Angle from unit vertically 'up' vector.
	const TVector normal(0, -1);
	return Angle(normal);
	}

///////////////////////////////////////////////////////////////////////////////
TTrigUnit TVector::Magnitude() const
	{
	// m = sqrt( (a.x * a.x) + (a.y * a.y) )
	TTrigUnit u(iX.Pow2());
	u += iY.Pow2();
	return u.Sqrt();
	}

///////////////////////////////////////////////////////////////////////////////
TDirection TVector::Angle(const TVector& aVector) const
	{
	// A = acos(a.b / |a|.|b|)
	TTrigUnit m(Magnitude());
	m *= aVector.Magnitude();
	m = ScalarProduct(aVector) / m;
	TDirection dir(Trigonometry::ACos(m));
	if(iX < 0)
		{
		dir += (TDirection::KResolution >> 1);
		}
	return dir;
	}

///////////////////////////////////////////////////////////////////////////////
TTrigUnit TVector::ScalarProduct(const TVector& aVector) const
	{
	// a.b = (a.x * b.x) + (a.y * b.y)
	TVector v(*this);
	v *= aVector;
	v.iX += v.iY;
	return v.iX;
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

