///////////////////////////////////////////////////////////////////////////////
//
// Trigonometry.cpp
//
//
//
//
//
///////////////////////////////////////////////////////////////////////////////
#include "Trigonometry.h"
#include <s32strm.h>
#include <barsread.h>

///////////////////////////////////////////////////////////////////////////////
// Constants.
///////////////////////////////////////////////////////////////////////////////
const TReal KTrigSinLookup[] =
	{
	0.00,
	0.382683432,
	0.707106781,
	0.923879532,
	1.00,
	0.923879532,
	0.707106781,
	0.382683432
	};

const TReal KTrigASinLookup[] =
	{
	0.00,
	0.125327831,
	0.252680255,
	0.384396774,
	0.523598776,
	0.675131533,
	0.848062079,
	1.065435817,
	1.570796327
	};

const TReal KTrigACosLookup[] =
	{
	3.141592654,
	2.636232143,
	2.418858406,
	2.245927860,
	2.094395102,
	1.955193101,
	1.823476582,
	1.696124158,
	1.570796327,
	1.445468496,
	1.318116072,
	1.186399552,
	1.047197551,
	0.895664794,
	0.722734248,
	0.505360510
	};
const TInt KTrigACosLookupSize = 16;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TTrigXY
///////////////////////////////////////////////////////////////////////////////
void TTrigXY::InternalizeL(RReadStream& aStream)
	{
	iX.InternalizeL(aStream);
	iY.InternalizeL(aStream);
	}

///////////////////////////////////////////////////////////////////////////////
void TTrigXY::ExternalizeL(RWriteStream& aStream) const
	{
	iX.ExternalizeL(aStream);
	iY.ExternalizeL(aStream);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TDirection
///////////////////////////////////////////////////////////////////////////////
TDirection::TDirection(TTrigUnit aAngle)
: iValue(0)
	{
	TTrigUnit u(Trigonometry::KPi());
	u /= KResolution;
	aAngle /= u;
	iValue = aAngle.NearestInt() / 2;
	}

///////////////////////////////////////////////////////////////////////////////
void TDirection::InternalizeL(TResourceReader& aReader)
	{
	iValue = aReader.ReadInt8();
	}

///////////////////////////////////////////////////////////////////////////////
void TDirection::InternalizeL(RReadStream& aStream)
	{
	iValue = aStream.ReadInt8L();
	}

///////////////////////////////////////////////////////////////////////////////
void TDirection::ExternalizeL(RWriteStream& aStream) const
	{
	aStream.WriteInt8L(iValue);
	}

///////////////////////////////////////////////////////////////////////////////
TDirection TDirection::Opposite() const
	{
	TDirection ret(iValue);
	ret.Reverse();
	return ret;
	}

///////////////////////////////////////////////////////////////////////////////
TTrigXY TDirection::Vector(const TTrigUnit& aMagnitude) const
	{
	TTrigXY v(Trigonometry::Sin(*this), -Trigonometry::Cos(*this));
	v *= aMagnitude;
	return v;
	}

///////////////////////////////////////////////////////////////////////////////
TDirection& TDirection::operator-=(TInt aSrc)
	{
	iValue -= aSrc;
	if(iValue < 0)
		{
		iValue += KResolution;
		}
	else
		{
		iValue %= KResolution;
		}
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
TDirection& TDirection::operator+=(TInt aSrc)
	{
	iValue += aSrc;
	if(iValue < 0)
		{
		iValue += KResolution;
		}
	else
		{
		iValue %= KResolution;
		}
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
TDirection TDirection::operator-(TInt aSrc) const
	{
	TDirection ret(iValue);
	ret -= aSrc;
	return ret;
	}

///////////////////////////////////////////////////////////////////////////////
TDirection TDirection::operator+(TInt aSrc) const
	{
	TDirection ret(iValue);
	ret += aSrc;
	return ret;
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Trigonometry
///////////////////////////////////////////////////////////////////////////////
TTrigUnit Trigonometry::Sin(TDirection aDir)
	{
	TInt sign = 1;
	TInt val = aDir.Value();
	const TInt resBy2 = (TDirection::KResolution >> 1);
	if(val > resBy2)
		{
		val %= resBy2;
		sign = -1;
		}
	const TTrigUnit u(KTrigSinLookup[val] * sign);
	return u;
	}

///////////////////////////////////////////////////////////////////////////////
TTrigUnit Trigonometry::Cos(TDirection aDir)
	{
	return Sin(aDir + (TDirection::KResolution >> 2));
	}

///////////////////////////////////////////////////////////////////////////////
TDirection Trigonometry::ASin(TTrigUnit aVal)
	{
	TTrigUnit val(Abs(aVal));
	TTrigUnit step(1);
	if(val > step)
		{
		return TDirection(0);
		}
	step.Set(0.125);
	val /= step;
	TInt i = val.NearestInt();
	step.Zero();
	val = TTrigUnit(KTrigASinLookup[i]);
	if(val < step)
		{
		val += KPi();
		}
	return TDirection(val);
	}

///////////////////////////////////////////////////////////////////////////////
TDirection Trigonometry::ACos(TTrigUnit aVal)
	{
	TTrigUnit val(aVal);
	TTrigUnit step(1);
	if(val > step)
		{
		return TDirection(0);
		}
	step.Set(0.125);
	val /= step;
	TInt i = val.NearestInt();
	i += (TDirection::KResolution >> 1);
	if(i < 0)
		i = 0;
	if(i >= KTrigACosLookupSize)
		i = KTrigACosLookupSize - 1;
	return TDirection(TTrigUnit(KTrigACosLookup[i]));
	}

///////////////////////////////////////////////////////////////////////////////
TTrigUnit Trigonometry::KPi()
	{
	return TTrigUnit(::KPi);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
