///////////////////////////////////////////////////////////////////////////////
//
// Trigonometry.inl
//
//
//
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef TRIGONOMETRY_INL
#define TRIGONOMETRY_INL

///////////////////////////////////////////////////////////////////////////////
// TTrigUnit
///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit::TTrigUnit()
: iValue(0.00)
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit::TTrigUnit(const TTrigUnit& aSrc)
: iValue(aSrc.iValue)
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit::TTrigUnit(TInt aInt)
: iValue(TReal(aInt))
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit::TTrigUnit(TReal aSrc)
: iValue(aSrc)
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigUnit::InternalizeL(RReadStream& aStream)
	{
	iValue = aStream.ReadReal64L();
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigUnit::ExternalizeL(RWriteStream& aStream) const
	{
	aStream.WriteReal64L(iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigUnit::Set(TInt aInt)
	{
	iValue = TReal(aInt);
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigUnit::Set(TReal aReal)
	{
	iValue = aReal;
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigUnit::Zero()
	{
	iValue = 0.00;
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigUnit::Format(TDes& aDes) const
	{
	aDes.Format(_L("%g"), iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TInt TTrigUnit::NearestInt() const
	{
	TInt32 r = 0;
	(void)Math::Int(r, iValue + 0.5);
	return r;
	}

///////////////////////////////////////////////////////////////////////////////
inline TInt TTrigUnit::Int() const
	{
	TInt32 r = 0;
	(void)Math::Int(r, iValue);
	return r;
	}

///////////////////////////////////////////////////////////////////////////////
inline TInt TTrigUnit::Fraction() const
	{
	TReal r = 0;
	(void)Math::Frac(r, iValue);
	return TInt(r * KMaxTInt32);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::Pow2() const
	{
	return TTrigUnit(iValue * iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::Sqrt() const
	{
	TTrigUnit r;
	(void)Math::Sqrt(r.iValue, iValue);
	return r;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit& TTrigUnit::operator=(const TTrigUnit& aSrc)
	{
	iValue = aSrc.iValue;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit& TTrigUnit::operator+=(const TTrigUnit& aSrc)
	{
	iValue += aSrc.iValue;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit& TTrigUnit::operator-=(const TTrigUnit& aSrc)
	{
	iValue -= aSrc.iValue;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit& TTrigUnit::operator*=(const TTrigUnit& aSrc)
	{
	iValue *= aSrc.iValue;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit& TTrigUnit::operator/=(const TTrigUnit& aSrc)
	{
	iValue /= aSrc.iValue;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit& TTrigUnit::operator%=(const TTrigUnit& aSrc)
	{
	const TReal val(iValue);
	(void)Math::Mod(iValue, val, aSrc.iValue);
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator+() const
	{
	return TTrigUnit(Abs(iValue));
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator-() const
	{
	return TTrigUnit( (iValue < 0) ? (Abs(iValue)) : (-iValue) );
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator+(const TTrigUnit& aSrc) const
	{
	TTrigUnit u(*this);
	u += aSrc;
	return u;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator-(const TTrigUnit& aSrc) const
	{
	TTrigUnit u(*this);
	u -= aSrc;
	return u;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator*(const TTrigUnit& aSrc) const
	{
	TTrigUnit u(*this);
	u *= aSrc;
	return u;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator/(const TTrigUnit& aSrc) const
	{
	TTrigUnit u(*this);
	u /= aSrc;
	return u;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigUnit TTrigUnit::operator%(const TTrigUnit& aSrc) const
	{
	TTrigUnit u(*this);
	u %= aSrc;
	return u;
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigUnit::operator==(const TTrigUnit& aSrc) const
	{
	return TBool(iValue == aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigUnit::operator!=(const TTrigUnit& aSrc) const
	{
	return TBool(iValue != aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigUnit::operator<=(const TTrigUnit& aSrc) const
	{
	return TBool(iValue <= aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigUnit::operator>=(const TTrigUnit& aSrc) const
	{
	return TBool(iValue >= aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigUnit::operator<(const TTrigUnit& aSrc) const
	{
	return TBool(iValue < aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigUnit::operator>(const TTrigUnit& aSrc) const
	{
	return TBool(iValue > aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TTrigXY
///////////////////////////////////////////////////////////////////////////////
inline TTrigXY::TTrigXY()
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY::TTrigXY(const TSize& aSize)
	{
	Set(aSize.iWidth, aSize.iHeight);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY::TTrigXY(const TPoint& aPoint)
	{
	Set(aPoint.iX, aPoint.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY::TTrigXY(TTrigUnit aX, TTrigUnit aY)
	{
	Set(aX, aY);
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigXY::Set(TTrigUnit aX, TTrigUnit aY)
	{
	iX = aX;
	iY = aY;
	}

///////////////////////////////////////////////////////////////////////////////
inline void TTrigXY::Zero()
	{
	iX.Zero();
	iY.Zero();
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator=(const TTrigXY& aSrc)
	{
	iX = aSrc.iX;
	iY = aSrc.iY;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator+=(const TTrigXY& aSrc)
	{
	iX += aSrc.iX;
	iY += aSrc.iY;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator-=(const TTrigXY& aSrc)
	{
	iX -= aSrc.iX;
	iY -= aSrc.iY;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator*=(const TTrigXY& aSrc)
	{
	iX *= aSrc.iX;
	iY *= aSrc.iY;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator/=(const TTrigXY& aSrc)
	{
	iX /= aSrc.iX;
	iY /= aSrc.iY;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator%=(const TTrigXY& aSrc)
	{
	iX %= aSrc.iX;
	iY %= aSrc.iY;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator=(TTrigUnit aSrc)
	{
	iX = aSrc;
	iY = aSrc;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator+=(TTrigUnit aSrc)
	{
	iX += aSrc;
	iY += aSrc;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator-=(TTrigUnit aSrc)
	{
	iX -= aSrc;
	iY -= aSrc;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator*=(TTrigUnit aSrc)
	{
	iX *= aSrc;
	iY *= aSrc;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator/=(TTrigUnit aSrc)
	{
	iX /= aSrc;
	iY /= aSrc;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY& TTrigXY::operator%=(TTrigUnit aSrc)
	{
	iX %= aSrc;
	iY %= aSrc;
	return *this;
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator+(const TTrigXY& aSrc) const
	{
	return TTrigXY(iX + aSrc.iX, iY + aSrc.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator-(const TTrigXY& aSrc) const
	{
	return TTrigXY(iX - aSrc.iX, iY - aSrc.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator*(const TTrigXY& aSrc) const
	{
	return TTrigXY(iX * aSrc.iX, iY * aSrc.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator/(const TTrigXY& aSrc) const
	{
	return TTrigXY(iX / aSrc.iX, iY / aSrc.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator%(const TTrigXY& aSrc) const
	{
	return TTrigXY(iX % aSrc.iX, iY % aSrc.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator+(TTrigUnit aSrc) const
	{
	return TTrigXY(iX + aSrc, iY + aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator-(TTrigUnit aSrc) const
	{
	return TTrigXY(iX - aSrc, iY - aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator*(TTrigUnit aSrc) const
	{
	return TTrigXY(iX * aSrc, iY * aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator/(TTrigUnit aSrc) const
	{
	return TTrigXY(iX / aSrc, iY / aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
inline TTrigXY TTrigXY::operator%(TTrigUnit aSrc) const
	{
	return TTrigXY(iX % aSrc, iY % aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigXY::operator==(const TTrigXY& aSrc) const
	{
	return (iX == aSrc.iX) && (iY == aSrc.iY);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TTrigXY::operator!=(const TTrigXY& aSrc) const
	{
	return !operator==(aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// TDirection
///////////////////////////////////////////////////////////////////////////////
inline TDirection::TDirection()
: iValue(0)
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection::TDirection(TInt aValue)
: iValue(aValue)
	{
	}

///////////////////////////////////////////////////////////////////////////////
inline void TDirection::Reverse()
	{
	operator+=(KResolution >> 1);
	}

///////////////////////////////////////////////////////////////////////////////
inline TInt TDirection::Value() const
	{
	return iValue;
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TDirection::operator==(const TDirection& aSrc) const
	{
	return TBool(iValue == aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TBool TDirection::operator!=(const TDirection& aSrc) const
	{
	return !operator==(aSrc);
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection& TDirection::operator-=(const TDirection& aSrc)
	{
	return operator-=(aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection& TDirection::operator+=(const TDirection& aSrc)
	{
	return operator+=(aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection TDirection::operator-(const TDirection& aSrc) const
	{
	return operator-(aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection TDirection::operator+(const TDirection& aSrc) const
	{
	return operator+(aSrc.iValue);
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection& TDirection::operator--()
	{
	return operator-=(1);
	}

///////////////////////////////////////////////////////////////////////////////
inline TDirection& TDirection::operator++()
	{
	return operator+=(1);
	}

///////////////////////////////////////////////////////////////////////////////
inline const TDirection TDirection::operator--(TInt)
	{
	return operator-(1);
	}

///////////////////////////////////////////////////////////////////////////////
inline const TDirection TDirection::operator++(TInt)
	{
	return operator+(1);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
